#include <gtk/gtk.h>

#include <stdio.h>
#include <string.h>

#include <dbui-defs.h>

#include <dbui-structs.h>

#include <libpq-fe.h>

#include <postgres-prototypes.h>

#include "../gui/function_prototypes.h"

GList *
postgres_exec_query (TABLE * database, TABLE * table, char *query)
{
  PGresult *result;
  CONNECTION *connect;
  COLUMN *col;
  GList *col_values;
  GList *results = NULL;
  char *field_name;
  char *value;
  int type;
  int num_rows;
  int num_fields;
  int i, j;
  int query_ok;

  connect = table->my_connect;

  printf ("Execing %s\n", query);
  fflush (stdout);

  result = PQexec(connect->handle,query);

  if(!result)
   {
   return NULL;
   }

   query_ok = PQresultStatus(result);

   if(query_ok != PGRES_COMMAND_OK && 
	query_ok != PGRES_TUPLES_OK)
	{
	printf("Error %s\n",PQresultErrorMessage(result));
	return NULL;
	}


  num_rows = PQntuples (result);

  num_fields = PQnfields (result);

  for (i = 0; i < num_rows; i++)
    {

      col_values = NULL;

      for (j = 0; j < num_fields; j++)
	{
	  field_name = PQfname(result,j);

	  type = PQftype(result,j);

	  col = create_new_col (field_name, table);

	  col->type = postgres_change_type_to_dbui (type);

	  col->size = PQfsize(result,j);

	  value = PQgetvalue(result,i,j);	  

	  if(value)
	    {
	      col->value = (char *) calloc (
					    (strlen (value) + 1),
					    sizeof (char));

	      strcpy (col->value, value);
	    }
	  else
	    col->value = NULL;

	  col_values = g_list_append (col_values, col);
	}

      results = g_list_append (results, col_values);

    }

  PQclear (result);

  return results;
}
